Package: ShortRead
Type: Package
Title: FASTQ input and manipulation
Version: 1.70.0
Authors@R: c(
        person("Bioconductor Package Maintainer",
                email = "maintainer@bioconductor.org", role = "cre"),
        person("Martin", "Morgan", role = "aut"),
        person("Michael", "Lawrence", role = "ctb"),
        person("Simon", "Anders", role = "ctb"),
        person("Rohit", "Satyam", role = "ctb",
                comment = "Converted Overview.Rnw vignette from Sweave to RMarkdown / HTML."),
	person("J", "Wokaty", role = "ctb")
    )
Description: This package implements sampling, iteration, and input of
        FASTQ files. The package includes functions for filtering and
        trimming reads, and for generating a quality assessment report.
        Data are represented as DNAStringSet-derived objects, and
        easily manipulated for a diversity of purposes.  The package
        also contains legacy support for early single-end, ungapped
        alignment formats.
License: Artistic-2.0
LazyLoad: yes
Depends: BiocGenerics (>= 0.23.3), BiocParallel, Biostrings (>=
        2.47.6), Rsamtools (>= 1.31.2), GenomicAlignments (>= 1.15.6)
Imports: Biobase, S4Vectors (>= 0.17.25), IRanges (>= 2.13.12),
        Seqinfo, GenomicRanges (>= 1.31.8), pwalign, hwriter, methods,
        lattice, latticeExtra,
Suggests: BiocStyle, RUnit, biomaRt, GenomicFeatures, yeastNagalakshmi,
        knitr
LinkingTo: S4Vectors, IRanges, XVector, Biostrings, Rhtslib
biocViews: DataImport, Sequencing, QualityControl
URL: https://bioconductor.org/packages/ShortRead,
        https://github.com/Bioconductor/ShortRead,
        https://support.bioconductor.org/tag/ShortRead
BugReports: https://github.com/Bioconductor/ShortRead/issues
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/ShortRead
git_branch: RELEASE_3_23
git_last_commit: ee5a413
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:42:04 UTC; biocbuild
Author: Bioconductor Package Maintainer [cre],
  Martin Morgan [aut],
  Michael Lawrence [ctb],
  Simon Anders [ctb],
  Rohit Satyam [ctb] (Converted Overview.Rnw vignette from Sweave to
    RMarkdown / HTML.),
  J Wokaty [ctb]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
